"""Const of iNels BUS."""

ATTR_SWITCH_OFF = "0"
ATTR_SWITCH_ON = "1"

NAME = "pyinels"

RANGE_BLIND = (0, 100)
RANGE_BRIGHTNESS = (0.0, 255.0)

SUPPORT_BRIGHTNESS = 1
SUPPORT_COLOR_TEMP = 4
SUPPORT_HEX_COLOR = 8
SUPPORT_RGB_COLOR = 2

VERSION = "0.2.1"
